=== modified file 'glib/glib-object.h'
--- glib/glib-object.h	2001-11-18 00:38:48 +0000
+++ glib/glib-object.h	2008-07-18 11:46:11 +0000
@@ -24,6 +24,7 @@
 /* topmost include file for GObject header files */
 #include	<gobject/gboxed.h>
 #include	<gobject/genums.h>
+#include	<gobject/giterable.h>
 #include	<gobject/gobject.h>
 #include	<gobject/gparam.h>
 #include	<gobject/gparamspecs.h>

=== modified file 'gobject/Makefile.am'
--- gobject/Makefile.am	2008-05-28 16:16:55 +0000
+++ gobject/Makefile.am	2008-07-18 11:29:39 +0000
@@ -91,6 +91,7 @@
 # GObject library header files for public installation
 gobject_public_h_sources = \
 	gboxed.h		\
+	giterable.h		\
 	gclosure.h		\
 	genums.h		\
 	gobject.h		\
@@ -113,6 +114,7 @@
 # GObject library C sources to build the library from
 gobject_c_sources = \
 	gboxed.c		\
+	giterable.c		\
 	gclosure.c		\
 	genums.c		\
 	gobject.c		\

=== added file 'gobject/giterable.c'
--- gobject/giterable.c	1970-01-01 00:00:00 +0000
+++ gobject/giterable.c	2008-07-18 12:49:21 +0000
@@ -0,0 +1,266 @@
+/* GObject - GLib Type, Object, Parameter and Signal Library
+ * Copyright (C) 2000-2001 Red Hat, Inc.
+ *
+ * giterable.h and giterable.c are written and copyright by
+ * Mikkel Kamstrup Erlandsen 2008
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
+#include "config.h"
+
+#include <string.h>
+
+#include "giterable.h"
+#include "gboxed.h"
+#include "gbsearcharray.h"
+#include "gvalue.h"
+#include "gvaluearray.h"
+#include "gclosure.h"
+#include "gvaluecollector.h"
+#include "gobjectalias.h"
+
+/*
+ * TODO
+ * 
+ * - Implement a GParamSpecIterable
+ * - Implement GSList, GList, GArray, and GPtrArray as GIterables
+ */
+
+/**
+ * SECTION:giterable
+ * @short_description: A mechanism to iterate over opaque data structures that
+ *                     expose a set-like interface
+ * @see_also: #GParamSpecIterable, g_param_spec_iterable()
+ * @title: Iterable Types
+ *
+ * GIterable is a generic wrapper mechanism for arbitrary C structures that
+ * model a set-like structure.
+ * The only thing the type system needs to know about the structures is how to
+ * copy and free them as well as how to iterate over its members, beyond that
+ * they are treated as opaque chunks of memory.
+ *
+ * Iterable types are useful for snafu
+ */
+
+/* --- typedefs & structures --- */
+typedef struct
+{
+  GType			type;
+  GIterNextFunc		next;
+  GIterHasNextFunc	has_next;
+  GIterBreakFunc	brake; /* don't use the C keyword 'break' */
+} IterableNode;
+
+/* --- prototypes --- */
+static gint	iterable_nodes_cmp	(gconstpointer	p1,
+					 gconstpointer	p2);
+
+
+/* --- variables --- */
+static GBSearchArray *iterable_bsa = NULL;
+static const GBSearchConfig iterable_bconfig = {
+  sizeof (IterableNode),
+  iterable_nodes_cmp,
+  0,
+};
+
+
+/* --- functions --- */
+static gint
+iterable_nodes_cmp	(gconstpointer p1,
+			 gconstpointer p2)
+{
+  const IterableNode *node1 = p1, *node2 = p2;
+
+  return G_BSEARCH_ARRAY_CMP (node1->type, node2->type);
+}
+
+/* Declared in gtype.h and invoked by g_type_init() */
+void
+g_iterable_type_init (void)
+{
+  static const GTypeInfo info = {
+    0,                          /* class_size */
+    NULL,                       /* base_init */
+    NULL,                       /* base_destroy */
+    NULL,                       /* class_init */
+    NULL,                       /* class_destroy */
+    NULL,                       /* class_data */
+    0,                          /* instance_size */
+    0,                          /* n_preallocs */
+    NULL,                       /* instance_init */
+    NULL,                       /* value_table */
+  };
+  const GTypeFundamentalInfo finfo = { G_TYPE_FLAG_DERIVABLE |
+				       G_TYPE_FLAG_DEEP_DERIVABLE, };
+  GType type;
+
+  iterable_bsa = g_bsearch_array_create (&iterable_bconfig);
+
+  /* G_TYPE_ITERABLE
+   */
+  type = g_type_register_fundamental (G_TYPE_ITERABLE, g_intern_static_string ("GIterable"), &info, &finfo,
+				      G_TYPE_FLAG_ABSTRACT | G_TYPE_FLAG_VALUE_ABSTRACT);
+  g_assert (type == G_TYPE_ITERABLE);
+}
+
+/**
+ * g_iterable_type_register_static:
+ * @name: Name of the new iterable type.
+ * @boxed_copy: Boxed structure copy function.
+ * @boxed_free: Boxed structure free function.
+ *
+ * This function creates a new %G_TYPE_ITERABLE derived type id for a new
+ * iterable type with name @name.
+ * 
+ * Returns: New %G_TYPE_ITERABLE derived type id for @name.
+ */
+GType
+g_iterable_type_register_static (const gchar		*name,
+				 GIterNextFunc	 	 next,
+				 GIterHasNextFunc	 has_next,
+				 GIterBreakFunc		 brake)
+{
+  static const GTypeInfo type_info = {
+    0,			/* class_size */
+    NULL,		/* base_init */
+    NULL,		/* base_finalize */
+    NULL,		/* class_init */
+    NULL,		/* class_finalize */
+    NULL,		/* class_data */
+    0,			/* instance_size */
+    0,			/* n_preallocs */
+    NULL,		/* instance_init */
+    NULL,		/* value_table */
+  };
+  GType type;
+
+  g_return_val_if_fail (name != NULL, 0);
+  g_return_val_if_fail (next != NULL, 0);
+  g_return_val_if_fail (has_next != NULL, 0);
+  g_return_val_if_fail (brake != NULL, 0);
+  g_return_val_if_fail (g_type_from_name (name) == 0, 0);
+
+  type = g_type_register_static (G_TYPE_ITERABLE, name, &type_info, 0);
+
+  /* install proxy functions upon successfull registration */
+  if (type)
+    {
+      IterableNode key;
+
+      key.type = type;
+      key.next = next;
+      key.has_next = has_next;
+      key.brake = brake;
+      iterable_bsa = g_bsearch_array_insert (iterable_bsa, &iterable_bconfig,
+					  &key);
+    }
+
+  return type;
+}
+
+gpointer
+g_iter_next (GType	 iterable_type,
+	     GIter 	*iter,
+	     gpointer	 iterable)
+{
+  IterableNode key, *node;
+  
+  g_return_val_if_fail (G_TYPE_IS_ITERABLE (iterable_type), NULL);
+  g_return_val_if_fail (G_TYPE_IS_ABSTRACT (iterable_type) == FALSE, NULL);
+  g_return_val_if_fail (iter != NULL, NULL);
+  g_return_val_if_fail (iterable != NULL, NULL);
+  
+  key.type = iterable_type;
+  node = g_bsearch_array_lookup (iterable_bsa, &iterable_bconfig, &key);
+  return node->next (iter, iterable);
+}
+
+gboolean
+g_iter_has_next (GType		 iterable_type,
+		 GIter		*iter,
+		 gpointer	 iterable)
+{
+  IterableNode key, *node;
+  
+  g_return_val_if_fail (G_TYPE_IS_ITERABLE (iterable_type), NULL);
+  g_return_val_if_fail (G_TYPE_IS_ABSTRACT (iterable_type) == FALSE, NULL);
+  g_return_val_if_fail (iter != NULL, NULL);
+  g_return_val_if_fail (iterable != NULL, NULL);
+  
+  key.type = iterable_type;
+  node = g_bsearch_array_lookup (iterable_bsa, &iterable_bconfig, &key);
+  return node->has_next (iter, iterable);  
+}
+
+void
+g_iter_break (GType	 iterable_type,
+	      GIter	*iter,
+	      gpointer	 iterable)
+{
+  IterableNode key, *node;
+  
+  g_return_val_if_fail (G_TYPE_IS_ITERABLE (iterable_type), NULL);
+  g_return_val_if_fail (G_TYPE_IS_ABSTRACT (iterable_type) == FALSE, NULL);
+  g_return_val_if_fail (iter != NULL, NULL);
+  g_return_val_if_fail (iterable != NULL, NULL);
+  
+  key.type = iterable_type;
+  node = g_bsearch_array_lookup (iterable_bsa, &iterable_bconfig, &key);
+  node->brake (iter, iterable);
+}
+
+gpointer
+g_iter_get_user_data (GIter	*iter)
+{
+  g_return_val_if_fail (iter != NULL, NULL);
+    
+  return iter->user_data;
+}
+
+gpointer
+g_iter_set_user_data (GIter	*iter,
+		      gpointer	 user_data)
+{
+  g_return_val_if_fail (iter != NULL, NULL);
+  
+  iter->user_data = user_data;
+    
+  return user_data;
+}
+
+gpointer
+g_iter_get_iter_data (GIter	*iter)
+{
+  g_return_val_if_fail (iter != NULL, NULL);
+    
+  return iter->iter_data;
+}
+
+gpointer
+g_iter_set_iter_data (GIter	*iter,
+		      gpointer	 iter_data)
+{
+  g_return_val_if_fail (iter != NULL, NULL);
+  
+  iter->iter_data = iter_data;
+    
+  return iter_data;
+}
+
+#define __G_ITERABLE_C__
+#include "gobjectaliasdef.c"

=== added file 'gobject/giterable.h'
--- gobject/giterable.h	1970-01-01 00:00:00 +0000
+++ gobject/giterable.h	2008-07-18 12:35:49 +0000
@@ -0,0 +1,107 @@
+/* GObject - GLib Type, Object, Parameter and Signal Library
+ * Copyright (C) 2000-2001 Red Hat, Inc.
+ *
+ * giterable.h and giterable.c are written and copyright by
+ * Mikkel Kamstrup Erlandsen 2008
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+#if !defined (__GLIB_GOBJECT_H_INSIDE__) && !defined (GOBJECT_COMPILATION)
+#error "Only <glib-object.h> can be included directly."
+#endif
+
+#ifndef __G_ITERABLE_H__
+#define __G_ITERABLE_H__
+
+#include        <gobject/gtype.h>
+
+G_BEGIN_DECLS
+
+/* --- type macros --- */
+#define G_TYPE_IS_ITERABLE(type)	   (G_TYPE_FUNDAMENTAL (type) == G_TYPE_ITERABLE)
+
+/* --- typedefs --- */
+typedef struct {
+  gpointer	user_data;
+  gpointer	iter_data;
+  gpointer	padding_1;
+  gpointer	padding_2;
+} GIter;
+
+/**
+ * GIterNextFunc:
+ * @iter:
+ * @iterable:
+ * 
+ * 
+ */
+typedef gpointer (*GIterNextFunc) (GIter *iter, gpointer iterable);
+
+/**
+ * GIterHasNextFunc:
+ * @iter:
+ * @iterable:
+ * 
+ * This function is provided by the user and should free the boxed
+ * structure passed.
+ */
+typedef gboolean (*GIterHasNextFunc) (GIter *iter, gpointer iterable);
+
+/**
+ * GIterBreakFunc:
+ * @iter:
+ * @iterable:
+ *
+ * 
+ */
+typedef void (*GIterBreakFunc) (GIter *iter, gpointer iterable);
+
+/* --- prototypes --- */
+gpointer	g_iter_next		(GType		 boxed_type,
+					 GIter		*iter,
+					 gpointer	 iterable);
+gboolean	g_iter_has_next		(GType		 boxed_type,
+					 GIter		*iter,
+					 gpointer	 iterable);
+void		g_iter_break		(GType		 boxed_type,
+					 GIter		*iter,
+					 gpointer	 iterable);
+gpointer	g_iter_get_user_data	(GIter		*iter);
+gpointer	g_iter_set_user_data	(GIter		*iter,
+					 gpointer	user_data);
+gpointer	g_iter_get_iter_data	(GIter		*iter);
+gpointer	g_iter_set_iter_data	(GIter		*iter,
+					 gpointer	iter_data);
+
+
+/* --- convenience --- */
+GType	g_iterable_type_register_static	(const gchar		*name,
+					 GIterNextFunc	 	 next,
+					 GIterHasNextFunc	 has_next,
+					 GIterBreakFunc		 brake);
+
+
+/* --- GLib iterable types --- */
+/**
+ * G_TYPE_SLIST:
+ * 
+ * The #GType for #GClosure.
+ */
+#define	G_TYPE_SLIST		(g_slist_get_type ())
+     
+G_END_DECLS
+
+#endif	/* __G_ITERABLE_H__ */

=== modified file 'gobject/gtype.c'
--- gobject/gtype.c	2008-07-02 03:43:13 +0000
+++ gobject/gtype.c	2008-07-18 11:26:35 +0000
@@ -4054,6 +4054,10 @@
    */
   g_boxed_type_init ();
   
+  /* G_TYPE_ITERABLE
+   */
+  g_iterable_type_init ();
+  
   /* G_TYPE_PARAM
    */
   g_param_type_init ();

=== modified file 'gobject/gtype.h'
--- gobject/gtype.h	2008-06-22 10:10:59 +0000
+++ gobject/gtype.h	2008-07-18 11:36:23 +0000
@@ -179,7 +179,12 @@
  * The fundamental type for #GObject.
  */
 #define G_TYPE_OBJECT			G_TYPE_MAKE_FUNDAMENTAL (20)
-
+/**
+ * G_TYPE_ITERABLE:
+ * 
+ * The fundamental type from which all iterable types are derived
+ */
+#define G_TYPE_ITERABLE			G_TYPE_MAKE_FUNDAMENTAL (21)
 
 /* Reserved fundamental type numbers to create new fundamental
  * type IDs with G_TYPE_MAKE_FUNDAMENTAL().
@@ -208,7 +213,7 @@
  * First fundamental type number to create a new fundamental type id with
  * G_TYPE_MAKE_FUNDAMENTAL() reserved for GLib.
  */
-#define G_TYPE_RESERVED_GLIB_FIRST	(21)
+#define G_TYPE_RESERVED_GLIB_FIRST	(22)
 /**
  * G_TYPE_RESERVED_GLIB_LAST:
  * 
@@ -1455,6 +1460,7 @@
 G_GNUC_INTERNAL void    g_enum_types_init       (void); /* sync with genums.c */
 G_GNUC_INTERNAL void    g_param_type_init       (void); /* sync with gparam.c */
 G_GNUC_INTERNAL void    g_boxed_type_init       (void); /* sync with gboxed.c */
+G_GNUC_INTERNAL void    g_iterable_type_init    (void); /* sync with giterable.c */
 G_GNUC_INTERNAL void    g_object_type_init      (void); /* sync with gobject.c */
 G_GNUC_INTERNAL void    g_param_spec_types_init (void); /* sync with gparamspecs.c */
 G_GNUC_INTERNAL void    g_value_transforms_init (void); /* sync with gvaluetransform.c */

=== modified file 'gobject/tests/Makefile.am'
--- gobject/tests/Makefile.am	2008-02-05 17:42:09 +0000
+++ gobject/tests/Makefile.am	2008-07-18 11:42:21 +0000
@@ -8,3 +8,7 @@
 TEST_PROGS             += threadtests
 threadtests_SOURCES	= threadtests.c
 threadtests_LDADD	= $(libgobject_LDADD)
+
+TEST_PROGS             += iterabletests
+iterabletests_SOURCES	= iterabletests.c
+iterabletests_LDADD	= $(libgobject_LDADD)

=== added file 'gobject/tests/iterabletests.c'
--- gobject/tests/iterabletests.c	1970-01-01 00:00:00 +0000
+++ gobject/tests/iterabletests.c	2008-07-18 13:03:04 +0000
@@ -0,0 +1,214 @@
+/* GLib testing framework examples and tests
+ * Copyright (C) 2008 Mikkel Kamstrup Erlandsen
+ * Authors: Mikkel Kamstrup Erlandsen
+ *
+ * This work is provided "as is"; redistribution and modification
+ * in whole or in part, in any medium, physical or electronic is
+ * permitted without restriction.
+ *
+ * This work is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ *
+ * In no event shall the authors or contributors be liable for any
+ * direct, indirect, incidental, special, exemplary, or consequential
+ * damages (including, but not limited to, procurement of substitute
+ * goods or services; loss of use, data, or profits; or business
+ * interruption) however caused and on any theory of liability, whether
+ * in contract, strict liability, or tort (including negligence or
+ * otherwise) arising in any way out of the use of this software, even
+ * if advised of the possibility of such damage.
+ */
+#include <glib.h>
+#include <glib-object.h>
+
+/* Shortcut to defining a test */
+#define TEST(func,iterable,type) g_test_add ("/GObject/iterable/"#func, Fixture, type, (void (*) (Fixture*, gconstpointer))setup_##iterable, test_##func, teardown_##iterable);
+
+/* Test fixture passed to all tests */
+typedef struct {
+  gpointer	iterable;
+  GIter		*iter;
+} Fixture;
+
+/*
+ * GType shortcuts. We must use this naming scheme for the macro magic of TEST()
+ * to work.
+ */
+GType custom_type = G_TYPE_INVALID;
+//GType slist_type = G_TYPE_SLIST;
+
+/*
+ * CUSTOM ITERABLE
+ *
+ * Here follows a *crappy* implementation of GStrv as an iterable
+ */
+typedef struct {
+  guint	idx;
+} CustomIterCtx;
+
+/* Poison element used to signal when an iteration should stop */
+const CustomIterCtx	 custom_terminator = {0,};
+const gchar 		*custom_data[] = {"foo", "bar", "qiz", NULL};
+
+static gpointer
+custom_next (GIter *iter, gpointer iterable)
+{
+  GStrv 	 strv = iterable;
+  gchar		*val;
+  CustomIterCtx	*ctx;
+  
+  ctx = g_iter_get_iter_data (iter);
+  
+  /* Start a new iteration if there is no iter_data */
+  if (ctx == NULL)
+    {
+      ctx = g_new0 (CustomIterCtx, 1);
+      ctx->idx = 0;
+      g_iter_set_iter_data (iter, ctx);
+    }
+  
+  if (ctx == &custom_terminator)
+    {
+      g_critical ("Iteration past array size");
+      return NULL;
+    }
+  
+  val = strv[ctx->idx];
+  ++ctx->idx;
+  
+  /* If this was the last element free ctx, and set iter_data to
+   * our terminator context */
+  if (ctx->idx == g_strv_length (strv))
+    {
+      g_free (ctx);
+      g_iter_set_iter_data (iter, &custom_terminator);
+    }
+  
+  return val;
+  
+}
+
+static gboolean
+custom_has_next (GIter *iter, gpointer iterable)
+{
+  GStrv 	 strv = iterable;
+  CustomIterCtx	*ctx;
+  
+  ctx = g_iter_get_iter_data (iter);
+  
+  /* Start a new iteration if there is no iter_data */
+  if (ctx == NULL)
+    {
+      ctx = g_new0 (CustomIterCtx, 1);
+      ctx->idx = 0;
+      g_iter_set_iter_data (iter, ctx);
+    }
+  
+  return (ctx != &custom_terminator &&
+	  ctx->idx < g_strv_length(strv));
+}
+
+static void
+custom_break (GIter *iter, gpointer iterable)
+{
+  CustomIterCtx	*ctx;
+  
+  ctx = g_iter_get_iter_data (iter);
+  g_assert (ctx != NULL);
+  
+  if (ctx != &custom_terminator)
+    {
+      g_free (ctx);
+      g_iter_set_iter_data (iter, &custom_terminator);
+    }
+}
+
+/*
+ * CUSTOM FIXTURE HANDLING
+ */
+
+static void
+setup_custom (Fixture		*fix,
+	      gconstpointer	 data)
+{
+  if (custom_type == G_TYPE_INVALID)
+    {
+      custom_type = g_iterable_type_register_static ("CustomGStrvIterable",
+						     custom_next,
+						     custom_has_next,
+						     custom_break);
+    }
+  
+  fix->iterable = custom_data;
+  fix->iter = g_new0 (GIter, 1);
+}
+
+static void
+teardown_custom (Fixture	*fix,
+		 gconstpointer  test_data)
+{
+  g_free (fix->iter);
+}
+
+/*
+ * TEST CASES
+ */
+
+/* If this test passes the fixture setup and teardown work */
+static void
+test_fixture (Fixture	*fix,
+	      GType	*iterable_type)
+{
+  g_assert (TRUE);
+  
+  g_debug ("Testing fixture for %s", g_type_name (*iterable_type));
+}
+
+static void
+test_values (Fixture	*fix,
+	     GType	*iterable_type)
+{
+  GType type = *iterable_type;
+  guint count = 0;
+  
+  g_assert (type == custom_type);
+  g_assert (fix->iterable == custom_data);
+  
+  
+  
+  while (g_iter_has_next (type, fix->iter, fix->iterable))
+    {
+      count++;
+      gchar *val = g_iter_next(type, fix->iter, fix->iterable);
+      switch (count)
+	{
+	  case 1 :
+	    g_assert_cmpstr ("foo", ==, val);
+	    break;
+	  case 2 :
+	    g_assert_cmpstr ("bar", ==, val);
+	    break;
+	  case 3 :
+	    g_assert_cmpstr ("qiz", ==, val);
+	    break;
+	  default :
+	    g_critical ("Iteration out of bounds");
+	}	       
+    }
+  
+  g_assert_cmpint (3, ==, count);
+}
+
+int
+main (int   argc,
+      char *argv[])
+{
+  g_test_init (&argc, &argv, NULL);
+  g_type_init ();
+
+  TEST (fixture, custom, &custom_type);
+  TEST (values, custom, &custom_type);
+  
+  return g_test_run();
+}

